@@ -146,7 +146,10 @@ class TPE(BaseAlgorithm):
         Seed to sample initial points and candidates points.
         Default: ``None``
     n_initial_points: int
-        Number of initial points randomly sampled.
+        Number of initial points randomly sampled. If new points
+        are requested and less than `n_initial_points` are observed,
+        the next points will also be sampled randomly instead of being
+        sampled from the parzen estimators.
         Default: ``20``
     n_ei_candidates: int
         Number of candidates points sampled for ei compute.
@@ -168,7 +171,7 @@ class TPE(BaseAlgorithm):
     """
 
     requires_type = None
-    requires_dist = None
+    requires_dist = "linear"
     requires_shape = "flattened"
 
     # pylint:disable=too-many-arguments
@@ -229,11 +232,12 @@ class TPE(BaseAlgorithm):
                 "uniform",
                 "reciprocal",
                 "int_uniform",
+                "int_reciprocal",
                 "choices",
             ]:
                 raise ValueError(
                     "TPE now only supports uniform, loguniform, uniform discrete "
-                    "and choices as prior."
+                    f"and choices as prior: {dimension.prior_name}"
                 )
 
             shape = dimension.shape
@@ -266,89 +270,125 @@ class TPE(BaseAlgorithm):
         self.seed_rng(state_dict["seed"])
         self.rng.set_state(state_dict["rng_state"])
 
-    def suggest(self, num=1):
+    def suggest(self, num=None):
         """Suggest a `num` of new sets of parameters. Randomly draw samples
         from the import space and return them.
 
+        Parameters
+        ----------
+        num: int, optional
+            Number of points to sample. If None, TPE will sample all random points at once, or a
+            single point if it is at the Bayesian Optimization stage.
         :param num: how many sets to be suggested.
 
         .. note:: New parameters must be compliant with the problem's domain
            `orion.algo.space.Space`.
         """
-        if num > 1:
-            raise ValueError("TPE should suggest only one point.")
+        # Only sample up to `n_initial_points` and after that only sample one at a time.
+        num = min(num, max(self.n_initial_points - self.n_suggested, 1))
 
         samples = []
-        if len(self._trials_info) < self.n_initial_points:
-            new_point = self.space.sample(
-                1, seed=tuple(self.rng.randint(0, 1000000, size=3))
-            )[0]
-            samples.append(new_point)
-        else:
-            point = []
-            below_points, above_points = self.split_trials()
+        candidates = []
+        while len(samples) < num and self.n_suggested < self.space.cardinality:
+            if candidates:
+                candidate = candidates.pop(0)
+                if candidate:
+                    self.register(candidate)
+                    samples.append(candidate)
+            elif self.n_observed < self.n_initial_points:
+                candidates = self._suggest_random(num)
+            else:
+                candidates = self._suggest_bo(max(num - len(samples), 0))
 
-            below_points = [flatten_dims(point, self.space) for point in below_points]
-            above_points = [flatten_dims(point, self.space) for point in above_points]
-            below_points = list(map(list, zip(*below_points)))
-            above_points = list(map(list, zip(*above_points)))
+            if not candidates:
+                break
 
-            idx = 0
-            for i, dimension in enumerate(self.space.values()):
+        if samples:
+            return samples
 
-                shape = dimension.shape
-                if not shape:
-                    shape = (1,)
+        return None
 
-                if dimension.type == "real":
-                    points = self._sample_real_dimension(
-                        dimension,
-                        shape[0],
-                        below_points[idx : idx + shape[0]],
-                        above_points[idx : idx + shape[0]],
-                    )
-                elif (
-                    dimension.type == "integer"
-                    and dimension.prior_name == "int_uniform"
-                ):
-                    points = self.sample_one_dimension(
-                        dimension,
-                        shape[0],
-                        below_points[idx : idx + shape[0]],
-                        above_points[idx : idx + shape[0]],
-                        self._sample_int_point,
-                    )
-                elif (
-                    dimension.type == "categorical"
-                    and dimension.prior_name == "choices"
-                ):
-                    points = self.sample_one_dimension(
-                        dimension,
-                        shape[0],
-                        below_points[idx : idx + shape[0]],
-                        above_points[idx : idx + shape[0]],
-                        self._sample_categorical_point,
-                    )
-                elif dimension.type == "fidelity":
-                    # fidelity dimension
-                    points = [point[i] for point in self.space.sample(num)]
-                else:
-                    raise NotImplementedError()
+    def _suggest(self, num, function):
+        points = []
 
-                if len(points) < shape[0]:
-                    logger.warning(
-                        "TPE failed to sample new point with configuration %s",
-                        self.configuration,
-                    )
-                    return None
+        ids = set(self._trials_info.keys())
+        while len(points) < num:
+            for candidate in function(num - len(points)):
+                candidate_id = self.get_id(candidate)
+                if candidate_id not in ids:
+                    ids.add(candidate_id)
+                    points.append(candidate)
 
-                idx += shape[0]
-                point += points
+                if len(ids) >= self.space.cardinality:
+                    return points
 
-            point = regroup_dims(point, self.space)
-            samples.append(point)
+        return points
 
-        return samples
+    def _suggest_random(self, num):
+        def sample(num):
+            return self.space.sample(
+                num, seed=tuple(self.rng.randint(0, 1000000, size=3))
+            )
+
+        return self._suggest(num, sample)
+
+    def _suggest_bo(self, num):
+        def suggest_bo(num):
+            return [self._suggest_one_bo() for _ in range(num)]
+
+        return self._suggest(num, suggest_bo)
+
+    def _suggest_one_bo(self):
+
+        point = []
+        below_points, above_points = self.split_trials()
+
+        below_points = list(map(list, zip(*below_points)))
+        above_points = list(map(list, zip(*above_points)))
+
+        idx = 0
+        for i, dimension in enumerate(self.space.values()):
+
+            shape = dimension.shape
+            if not shape:
+                shape = (1,)
+
+            if dimension.type == "real":
+                dim_samples = self._sample_real_dimension(
+                    dimension,
+                    shape[0],
+                    below_points[idx : idx + shape[0]],
+                    above_points[idx : idx + shape[0]],
+                )
+            elif dimension.type == "integer" and dimension.prior_name in [
+                "int_uniform",
+                "int_reciprocal",
+            ]:
+                dim_samples = self.sample_one_dimension(
+                    dimension,
+                    shape[0],
+                    below_points[idx : idx + shape[0]],
+                    above_points[idx : idx + shape[0]],
+                    self._sample_int_point,
+                )
+            elif dimension.type == "categorical" and dimension.prior_name == "choices":
+                dim_samples = self.sample_one_dimension(
+                    dimension,
+                    shape[0],
+                    below_points[idx : idx + shape[0]],
+                    above_points[idx : idx + shape[0]],
+                    self._sample_categorical_point,
+                )
+            elif dimension.type == "fidelity":
+                # fidelity dimension
+                dim_samples = [point[i] for point in self.space.sample(1)]
+            else:
+                raise NotImplementedError()
+
+            idx += shape[0]
+            point += dim_samples
+
+        return self.format_point(point)
 
     # pylint:disable=no-self-use
     def sample_one_dimension(
@@ -366,14 +406,13 @@ class TPE(BaseAlgorithm):
 
         for j in range(shape_size):
             new_point = sampler(dimension, below_points[j], above_points[j])
-            if new_point is not None:
-                points.append(new_point)
+            points.append(new_point)
 
         return points
 
     def _sample_real_dimension(self, dimension, shape_size, below_points, above_points):
         """Sample values for real dimension"""
-        if dimension.prior_name == "uniform":
+        if dimension.prior_name in ["uniform", "reciprocal"]:
             return self.sample_one_dimension(
                 dimension,
                 shape_size,
@@ -381,14 +420,6 @@ class TPE(BaseAlgorithm):
                 above_points,
                 self._sample_real_point,
             )
-        elif dimension.prior_name == "reciprocal":
-            return self.sample_one_dimension(
-                dimension,
-                shape_size,
-                below_points,
-                above_points,
-                self._sample_loguniform_real_point,
-            )
         else:
             raise NotImplementedError()
 
@@ -432,22 +463,19 @@ class TPE(BaseAlgorithm):
         )
 
         candidate_points = gmm_sampler_below.sample(self.n_ei_candidates)
-        if candidate_points:
-            lik_blow = gmm_sampler_below.get_loglikelis(candidate_points)
-            lik_above = gmm_sampler_above.get_loglikelis(candidate_points)
-            new_point = compute_max_ei_point(candidate_points, lik_blow, lik_above)
+        lik_blow = gmm_sampler_below.get_loglikelis(candidate_points)
+        lik_above = gmm_sampler_above.get_loglikelis(candidate_points)
+        new_point = compute_max_ei_point(candidate_points, lik_blow, lik_above)
 
-            if is_log:
-                new_point = numpy.exp(new_point)
+        if is_log:
+            new_point = numpy.exp(new_point)
 
-            return new_point
-
-        return None
+        return new_point
 
     def _sample_int_point(self, dimension, below_points, above_points):
         """Sample one value for integer dimension based on the observed good and bad points"""
         low, high = dimension.interval()
-        choices = range(low, high)
+        choices = range(low, high + 1)
 
         below_points = numpy.array(below_points).astype(int) - low
         above_points = numpy.array(above_points).astype(int) - low
@@ -455,17 +483,14 @@ class TPE(BaseAlgorithm):
         sampler_below = CategoricalSampler(self, below_points, choices)
         candidate_points = sampler_below.sample(self.n_ei_candidates)
 
-        if list(candidate_points):
-            sampler_above = CategoricalSampler(self, above_points, choices)
+        sampler_above = CategoricalSampler(self, above_points, choices)
 
-            lik_below = sampler_below.get_loglikelis(candidate_points)
-            lik_above = sampler_above.get_loglikelis(candidate_points)
+        lik_below = sampler_below.get_loglikelis(candidate_points)
+        lik_above = sampler_above.get_loglikelis(candidate_points)
 
-            new_point = compute_max_ei_point(candidate_points, lik_below, lik_above)
-            new_point = new_point + low
-            return new_point
-
-        return None
+        new_point = compute_max_ei_point(candidate_points, lik_below, lik_above)
+        new_point = new_point + low
+        return new_point
 
     def _sample_categorical_point(self, dimension, below_points, above_points):
         """Sample one value for categorical dimension based on the observed good and bad points"""
@@ -477,24 +502,21 @@ class TPE(BaseAlgorithm):
         sampler_below = CategoricalSampler(self, below_points, choices)
         candidate_points = sampler_below.sample(self.n_ei_candidates)
 
-        if list(candidate_points):
-            sampler_above = CategoricalSampler(self, above_points, choices)
+        sampler_above = CategoricalSampler(self, above_points, choices)
 
-            lik_below = sampler_below.get_loglikelis(candidate_points)
-            lik_above = sampler_above.get_loglikelis(candidate_points)
+        lik_below = sampler_below.get_loglikelis(candidate_points)
+        lik_above = sampler_above.get_loglikelis(candidate_points)
 
-            new_point_index = compute_max_ei_point(
-                candidate_points, lik_below, lik_above
-            )
-            new_point = choices[new_point_index]
+        new_point_index = compute_max_ei_point(candidate_points, lik_below, lik_above)
+        new_point = choices[new_point_index]
 
-            return new_point
-        return None
+        return new_point
 
     def split_trials(self):
         """Split the observed trials into good and bad ones based on the ratio `gamma``"""
         sorted_trials = sorted(
-            self._trials_info.values(), key=lambda x: x[1]["objective"]
+            (point for point in self._trials_info.values() if point[1] is not None),
+            key=lambda x: x[1]["objective"],
         )
         sorted_points = [list(points) for points, results in sorted_trials]
 
@@ -553,14 +575,21 @@ class GMMSampler:
         for mu, sigma in zip(self.mus, self.sigmas):
             self.pdfs.append(norm(mu, sigma))
 
-    def sample(self, num=1):
+    def sample(self, num=1, attempts=10):
         """Sample required number of points"""
         point = []
         for _ in range(num):
             pdf = numpy.argmax(self.tpe.rng.multinomial(1, self.weights))
-            new_points = self.pdfs[pdf].rvs(size=5)
-            for pt in new_points:
-                if self.low <= pt < self.high:
+            new_points = list(
+                self.pdfs[pdf].rvs(size=attempts, random_state=self.tpe.rng)
+            )
+            while True:
+                if not new_points:
+                    raise RuntimeError(
+                        f"Failed to sample in interval ({self.low}, {self.high})"
+                    )
+                pt = new_points.pop(0)
+                if self.low <= pt <= self.high:
                     point.append(pt)
                     break
 
