@@ -335,7 +335,9 @@ def workon(
         producer = Producer(experiment)
 
         experiment_client = ExperimentClient(experiment, producer)
-        experiment_client.workon(function, max_trials=max_trials)
+        with experiment_client.tmp_executor("singleexecutor", n_workers=1):
+            experiment_client.workon(function, n_workers=1, max_trials=max_trials)
+
     finally:
         # Restore singletons
         update_singletons(singletons)
