@@ -1,12 +1,10 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 """
-:mod:`orion.core.cli` -- Functions that define console scripts
-================================================================
+Functions that define console scripts
+=====================================
 
-.. module:: cli
-   :platform: Unix
-   :synopsis: Helper functions to setup an experiment and execute it.
+Helper functions to setup an experiment and execute it.
 
 """
 import logging
@@ -19,10 +17,11 @@ log = logging.getLogger(__name__)
 
 def load_modules_parser(orion_parser):
     """Search through the `cli` folder for any module containing a `get_parser` function"""
-    modules = module_import.load_modules_in_path('orion.core.cli',
-                                                 lambda m: hasattr(m, 'add_subparser'))
+    modules = module_import.load_modules_in_path(
+        "orion.core.cli", lambda m: hasattr(m, "add_subparser")
+    )
     for module in modules:
-        get_parser = getattr(module, 'add_subparser')
+        get_parser = getattr(module, "add_subparser")
         get_parser(orion_parser.get_subparsers())
 
 
