@@ -1,12 +1,10 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 """
-:mod:`orion.core.cli.checks.creation` -- Creation stage for database checks
-===========================================================================
+Creation stage for database checks
+==================================
 
-.. module:: creation
-    :platform: Unix
-    :synopsis: Checks for the creation of a `Database` object.
+Checks for the creation of a `Database` object.
 
 """
 
@@ -36,7 +34,7 @@ class CreationStage:
     def check_database_creation(self):
         """Check if database of specified type can be created."""
         database = self.p_stage.db_config
-        db_type = database.pop('type')
+        db_type = database.pop("type")
 
         try:
             db = Database(of_type=db_type, **database)
@@ -52,4 +50,4 @@ class CreationStage:
 
     def post_stage(self):
         """Print the created database."""
-        print('DB instance', self.instance)
+        print("DB instance", self.instance)
