@@ -1169,9 +1169,15 @@ class CodeConflict(Conflict):
         old_hash_commit = old_config["metadata"].get("VCS", None)
         new_hash_commit = new_config["metadata"].get("VCS")
 
-        ignore_code_changes = branching_config is not None and branching_config.get(
-            "ignore_code_changes", False
-        )
+        # Will be overriden by global config if not set in branching_config
+        ignore_code_changes = None
+        # Try using user defined ignore_code_changes
+        if branching_config is not None:
+            ignore_code_changes = branching_config.get("ignore_code_changes", None)
+        # Otherwise use global conf's ignore_code_changes
+        if ignore_code_changes is None:
+            ignore_code_changes = orion.core.config.evc.ignore_code_changes
+
         if ignore_code_changes:
             log.debug("Ignoring code changes")
         if (
@@ -1318,7 +1324,7 @@ class CommandLineConflict(Conflict):
         log.debug("User script config: %s", user_script_config)
         log.debug("Non monitored arguments: %s", non_monitored_arguments)
 
-        parser = OrionCmdlineParser(user_script_config)
+        parser = OrionCmdlineParser(user_script_config, allow_non_existing_files=True)
         parser.set_state_dict(config["metadata"]["parser"])
         priors = parser.priors_to_normal()
         nameless_keys = set(parser.parser.arguments.keys()) - set(priors.keys())
@@ -1478,7 +1484,7 @@ class ScriptConfigConflict(Conflict):
         if user_script_config is None:
             user_script_config = orion.core.config.worker.user_script_config
 
-        parser = OrionCmdlineParser(user_script_config)
+        parser = OrionCmdlineParser(user_script_config, allow_non_existing_files=True)
         parser.set_state_dict(config["metadata"]["parser"])
 
         nameless_config = dict(
