@@ -1,11 +1,9 @@
 # -*- coding: utf-8 -*-
 """
-:mod:`orion.core.utils.diff` -- Utilitary functions for diffs
-=============================================================
+Utilitary functions for diffs
+=============================
 
-.. module:: experiment
-   :platform: Unix
-   :synopsis: Utilitary functions to provided colored diffs of (multi-line) strings
+Utilitary functions to provided colored diffs of (multi-line) strings
 
 """
 
@@ -35,7 +33,7 @@ def colored_diff(string_a, string_b):
 
     colored_result = []
     for i, line in enumerate(result):
-        line = line.strip('\n')
+        line = line.strip("\n")
         if line[0] == "-" or (line[0] == "?" and result[i - 1][0] == "-"):
             line = red(line)
         elif line[0] == "+" or (line[0] == "?" and result[i - 1][0] == "+"):
