@@ -16,8 +16,10 @@ import pandas
 
 from orion.core.evc.adapters import BaseAdapter
 from orion.core.evc.experiment import ExperimentNode
+from orion.core.io.database import DuplicateKeyError
 from orion.core.utils.exceptions import UnsupportedOperation
 from orion.core.utils.flatten import flatten
+from orion.core.utils.singleton import update_singletons
 from orion.storage.base import FailedUpdate, get_storage
 
 log = logging.getLogger(__name__)
@@ -97,8 +99,8 @@ class Experiment:
         "producer",
         "working_dir",
         "_id",
-        "_node",
         "_storage",
+        "_node",
         "_mode",
     )
     non_branching_attrs = ("pool_size", "max_trials", "max_broken")
@@ -118,6 +120,7 @@ class Experiment:
         self.algorithms = None
         self.working_dir = None
         self.producer = {}
+
         self._storage = get_storage()
 
         self._node = ExperimentNode(self.name, self.version, experiment=self)
@@ -136,6 +139,26 @@ class Experiment:
                 f"Experiment must have execution rights to execute `{calling_function}()`"
             )
 
+    def __getstate__(self):
+        """Remove storage instance during experiment serialization"""
+        state = dict()
+        for entry in self.__slots__:
+            state[entry] = getattr(self, entry)
+
+        # TODO: This should be removed when singletons and `get_storage()` are removed.
+        #       See https://github.com/Epistimio/orion/issues/606
+        singletons = update_singletons()
+        state["singletons"] = singletons
+        update_singletons(singletons)
+
+        return state
+
+    def __setstate__(self, state):
+        for entry in self.__slots__:
+            setattr(self, entry, state[entry])
+
+        update_singletons(state.pop("singletons"))
+
     def to_pandas(self, with_evc_tree=False):
         """Builds a dataframe with the trials of the experiment
 
@@ -218,11 +241,13 @@ class Experiment:
 
         self.fix_lost_trials()
 
+        self.duplicate_pending_trials()
+
         selected_trial = self._storage.reserve_trial(self)
         log.debug("reserved trial (trial: %s)", selected_trial)
         return selected_trial
 
-    def fix_lost_trials(self):
+    def fix_lost_trials(self, with_evc_tree=True):
         """Find lost trials and set them to interrupted.
 
         A lost trial is defined as a trial whose heartbeat as not been updated since two times
@@ -232,7 +257,18 @@ class Experiment:
 
         """
         self._check_if_writable()
-        trials = self._storage.fetch_lost_trials(self)
+
+        if self._node is not None and with_evc_tree:
+            for experiment in self._node.root:
+                if experiment.item is self:
+                    continue
+
+                # Ugly hack to allow resetting parent's lost trials.
+                experiment.item._mode = "w"
+                experiment.item.fix_lost_trials(with_evc_tree=False)
+                experiment.item._mode = "r"
+
+        trials = self.fetch_lost_trials(with_evc_tree=False)
 
         for trial in trials:
             log.debug("Setting lost trial %s status to interrupted...", trial.id)
@@ -243,6 +279,44 @@ class Experiment:
             except FailedUpdate:
                 log.debug("failed")
 
+    def duplicate_pending_trials(self):
+        """Find pending trials in EVC and duplicate them in current experiment.
+
+        An experiment cannot execute trials from parent experiments otherwise some trials
+        may have been executed in different environements of different experiment although they
+        belong to the same experiment. Instead, trials that are pending in parent and child
+        experiment are copied over to current experiment so that it can be reserved and executed.
+        The parent or child experiment will only see their original copy of the trial, and
+        the current experiment will only see the new copy of the trial.
+        """
+        self._check_if_writable()
+        evc_pending_trials = self._select_evc_call(
+            with_evc_tree=True, function="fetch_pending_trials"
+        )
+        exp_pending_trials = self._select_evc_call(
+            with_evc_tree=False, function="fetch_pending_trials"
+        )
+
+        exp_trials_ids = set(
+            trial.compute_trial_hash(trial, ignore_experiment=True)
+            for trial in exp_pending_trials
+        )
+
+        for trial in evc_pending_trials:
+            if (
+                trial.compute_trial_hash(trial, ignore_experiment=True)
+                in exp_trials_ids
+            ):
+                continue
+
+            trial.experiment = self.id
+            # Danger danger, race conditions!
+            try:
+                self._storage.register_trial(trial)
+            except DuplicateKeyError:
+                log.debug("Race condition while trying to duplicate trial %s", trial.id)
+
+    # pylint:disable=unused-argument
     def update_completed_trial(self, trial, results_file=None):
         """Inform database about an evaluated `trial` with results.
 
@@ -257,7 +331,7 @@ class Experiment:
         self._check_if_executable()
         trial.status = "completed"
         trial.end_time = datetime.datetime.utcnow()
-        self._storage.retrieve_result(trial, results_file)
+        self._storage.retrieve_result(trial)
         # push trial results updates the entire trial status included
         log.info("Completed trials with results: %s", trial.results)
         self._storage.push_trial_results(trial)
@@ -331,6 +405,24 @@ class Experiment:
         """
         return self._select_evc_call(with_evc_tree, "fetch_trials_by_status", status)
 
+    def fetch_pending_trials(self, with_evc_tree=False):
+        """Fetch all trials with status new, interrupted or suspended
+
+        Trials are sorted based on `Trial.submit_time`
+
+        :return: list of `Trial` objects
+        """
+        return self._select_evc_call(with_evc_tree, "fetch_pending_trials")
+
+    def fetch_lost_trials(self, with_evc_tree=False):
+        """Fetch all reserved trials that are lost (old heartbeat)
+
+        Trials are sorted based on `Trial.submit_time`
+
+        :return: list of `Trial` objects
+        """
+        return self._select_evc_call(with_evc_tree, "fetch_lost_trials")
+
     def fetch_noncompleted_trials(self, with_evc_tree=False):
         """Fetch non-completed trials of this `Experiment` instance.
 
