@@ -1,11 +1,7 @@
 # -*- coding: utf-8 -*-
 """
-:mod:`orion.serving.root_resource` -- Represent the '/' REST endpoint
-=====================================================================================
-
-.. module:: root_resource
-   :platform: Unix
-   :synopsis: Represent the '/' REST endpoint
+Represent the '/' REST endpoint
+===============================
 """
 import json
 
@@ -31,9 +27,9 @@ class RuntimeResource(object):
         """
         database = get_storage()._db.__class__.__name__
         response = {
-            'orion': orion.core.__version__,
-            'server': 'gunicorn',
-            'database': database
+            "orion": orion.core.__version__,
+            "server": "gunicorn",
+            "database": database,
         }
 
         resp.body = json.dumps(response, indent=4)
