@@ -3,6 +3,8 @@ import copy
 import datetime
 import os
 import random
+import shutil
+import tempfile
 from contextlib import contextmanager
 
 import pytest
@@ -10,9 +12,9 @@ import yaml
 
 import orion.core
 import orion.core.cli
+import orion.core.cli.hunt
 import orion.core.evc.conflicts
 import orion.core.io.resolve_config
-import orion.core.worker
 from orion.client import get_experiment
 from orion.core.io.database.pickleddb import PickledDB
 from orion.core.utils.singleton import SingletonNotInstantiatedError, update_singletons
@@ -25,6 +27,30 @@ script = os.path.join(
 )
 
 
+def with_storage_fork(func):
+    """Copy PickledDB to a tmp adress and work in the tmp path within the func execution.
+
+    Functions decorated with this decorator should only be called after the storage has been
+    initialized.
+    """
+
+    def call(*args, **kwargs):
+
+        with tempfile.NamedTemporaryFile(delete=True) as tmp_file:
+            storage = get_storage()
+            old_path = storage._db.host
+            storage._db.host = tmp_file.name
+            shutil.copyfile(old_path, tmp_file.name)
+
+            rval = func(*args, **kwargs)
+
+            storage._db.host = old_path
+
+        return rval
+
+    return call
+
+
 class ConfigurationTestSuite:
     """Test suite for the configuration groups"""
 
@@ -113,7 +139,7 @@ class ConfigurationTestSuite:
         with self.setup_db_config(tmp_path):
             self.check_db_config()
 
-    @pytest.mark.usefixtures("with_user_userxyz")
+    @pytest.mark.usefixtures("with_user_userxyz", "version_XYZ")
     def test_local_config(self, tmp_path, monkeypatch):
         """Test that local config overrides db/global config"""
         update_singletons()
@@ -179,7 +205,7 @@ class TestStorage(ConfigurationTestSuite):
         storage = get_storage()
         assert isinstance(storage, Legacy)
         assert isinstance(storage._db, PickledDB)
-        assert storage._db.host == "here.pkl"
+        assert storage._db.host == os.path.abspath("here.pkl")
 
     def check_env_var_config(self, tmp_path, monkeypatch):
         """Check that env vars overrides global configuration"""
@@ -204,7 +230,7 @@ class TestStorage(ConfigurationTestSuite):
         storage = get_storage()
         assert isinstance(storage, Legacy)
         assert isinstance(storage._db, PickledDB)
-        assert storage._db.host == self.env_vars["ORION_DB_ADDRESS"]
+        assert storage._db.host == os.path.abspath(self.env_vars["ORION_DB_ADDRESS"])
 
     def check_db_config(self):
         """No Storage config in DB, no test"""
@@ -233,7 +259,7 @@ class TestStorage(ConfigurationTestSuite):
         storage = get_storage()
         assert isinstance(storage, Legacy)
         assert isinstance(storage._db, PickledDB)
-        assert storage._db.host == "local.pkl"
+        assert storage._db.host == os.path.abspath("local.pkl")
 
     def check_cmd_args_config(self, tmp_path, conf_file, monkeypatch):
         """No Storage config in cmdline, no test"""
@@ -280,7 +306,7 @@ class TestDatabaseDeprecated(ConfigurationTestSuite):
         storage = get_storage()
         assert isinstance(storage, Legacy)
         assert isinstance(storage._db, PickledDB)
-        assert storage._db.host == "dbhere.pkl"
+        assert storage._db.host == os.path.abspath("dbhere.pkl")
 
     def check_env_var_config(self, tmp_path, monkeypatch):
         """Check that env vars overrides global configuration"""
@@ -302,7 +328,7 @@ class TestDatabaseDeprecated(ConfigurationTestSuite):
         storage = get_storage()
         assert isinstance(storage, Legacy)
         assert isinstance(storage._db, PickledDB)
-        assert storage._db.host == self.env_vars["ORION_DB_ADDRESS"]
+        assert storage._db.host == os.path.abspath(self.env_vars["ORION_DB_ADDRESS"])
 
     def check_db_config(self):
         """No Storage config in DB, no test"""
@@ -328,7 +354,7 @@ class TestDatabaseDeprecated(ConfigurationTestSuite):
         storage = get_storage()
         assert isinstance(storage, Legacy)
         assert isinstance(storage._db, PickledDB)
-        assert storage._db.host == "dblocal.pkl"
+        assert storage._db.host == os.path.abspath("dblocal.pkl")
 
     def check_cmd_args_config(self, tmp_path, conf_file, monkeypatch):
         """No Storage config in cmdline, no test"""
@@ -513,7 +539,7 @@ class TestExperimentConfig(ConfigurationTestSuite):
 
     def check_cmd_args_config(self, tmp_path, conf_file, monkeypatch):
         """Check that cmdargs configuration overrides global/envvars/local configuration"""
-        command = f"hunt --worker-max-trials 0 -c {conf_file} --branch-from test-name"
+        command = f"hunt --worker-max-trials 0 -c {conf_file} --branch-from test-name --enable-evc"
         command += " " + " ".join(
             "--{} {}".format(name, value) for name, value in self.cmdargs.items()
         )
@@ -536,6 +562,9 @@ class TestWorkerConfig(ConfigurationTestSuite):
 
     config = {
         "worker": {
+            "n_workers": 2,
+            "executor": "dask",
+            "executor_configuration": {"threads_per_worker": 1},
             "heartbeat": 30,
             "max_trials": 10,
             "max_broken": 5,
@@ -546,6 +575,8 @@ class TestWorkerConfig(ConfigurationTestSuite):
     }
 
     env_vars = {
+        "ORION_N_WORKERS": 3,
+        "ORION_EXECUTOR": "joblib",
         "ORION_HEARTBEAT": 40,
         "ORION_WORKER_MAX_TRIALS": 20,
         "ORION_WORKER_MAX_BROKEN": 6,
@@ -556,6 +587,9 @@ class TestWorkerConfig(ConfigurationTestSuite):
 
     local = {
         "worker": {
+            "n_workers": 4,
+            "executor": "dask",
+            "executor_configuration": {"threads_per_worker": 2},
             "heartbeat": 50,
             "max_trials": 30,
             "max_broken": 7,
@@ -566,8 +600,10 @@ class TestWorkerConfig(ConfigurationTestSuite):
     }
 
     cmdargs = {
+        "n-workers": 1,
+        "executor": "dask",
         "heartbeat": 70,
-        "worker-max-trials": 40,
+        "worker-max-trials": 1,
         "worker-max-broken": 8,
         "max-idle-time": 18,
         "interrupt-signal-code": 134,
@@ -578,28 +614,44 @@ class TestWorkerConfig(ConfigurationTestSuite):
         """Check that defaults are different than testing configuration"""
         assert orion.core.config.to_dict()["worker"] != self.config["worker"]
 
+    def _mock(self, monkeypatch):
+        self._mock_exp_client(monkeypatch)
+        self._mock_consumer(monkeypatch)
+        self._mock_producer(monkeypatch)
+        self._mock_workon(monkeypatch)
+
+    def _mock_exp_client(self, monkeypatch):
+        self.exp_client = None
+        old_init = orion.client.experiment.ExperimentClient.__init__
+
+        def init(c_self, *args, **kwargs):
+            old_init(c_self, *args, **kwargs)
+            self.exp_client = c_self
+
+        monkeypatch.setattr(orion.client.experiment.ExperimentClient, "__init__", init)
+
     def _mock_consumer(self, monkeypatch):
         self.consumer = None
-        old_init = orion.core.worker.Consumer.__init__
+        old_init = orion.core.cli.hunt.Consumer.__init__
 
         def init(c_self, *args, **kwargs):
             old_init(c_self, *args, **kwargs)
             self.consumer = c_self
 
-        monkeypatch.setattr(orion.core.worker.Consumer, "__init__", init)
+        monkeypatch.setattr(orion.core.cli.hunt.Consumer, "__init__", init)
 
     def _mock_producer(self, monkeypatch):
         self.producer = None
-        old_init = orion.core.worker.Producer.__init__
+        old_init = orion.core.cli.hunt.Producer.__init__
 
         def init(p_self, *args, **kwargs):
             old_init(p_self, *args, **kwargs)
             self.producer = p_self
 
-        monkeypatch.setattr(orion.core.worker.Producer, "__init__", init)
+        monkeypatch.setattr(orion.core.cli.hunt.Producer, "__init__", init)
 
     def _mock_workon(self, monkeypatch):
-        workon = orion.core.worker.workon
+        workon = orion.core.cli.hunt.workon
 
         self.workon_kwargs = None
 
@@ -609,8 +661,16 @@ class TestWorkerConfig(ConfigurationTestSuite):
 
         monkeypatch.setattr("orion.core.cli.hunt.workon", mocked_workon)
 
+    def _check_mocks(self, config):
+        self._check_exp_client(config)
+        self._check_consumer(config)
+        self._check_producer(config)
+        self._check_workon(config)
+
+    def _check_exp_client(self, config):
+        assert self.exp_client.heartbeat == config["heartbeat"]
+
     def _check_consumer(self, config):
-        assert self.consumer.heartbeat == config["heartbeat"]
         assert (
             self.consumer.template_builder.config_prefix == config["user_script_config"]
         )
@@ -620,6 +680,12 @@ class TestWorkerConfig(ConfigurationTestSuite):
         assert self.producer.max_idle_time == config["max_idle_time"]
 
     def _check_workon(self, config):
+        assert self.workon_kwargs["n_workers"] == config["n_workers"]
+        assert self.workon_kwargs["executor"] == config["executor"]
+        assert (
+            self.workon_kwargs["executor_configuration"]
+            == config["executor_configuration"]
+        )
         assert self.workon_kwargs["max_trials"] == config["max_trials"]
         assert self.workon_kwargs["max_broken"] == config["max_broken"]
 
@@ -627,20 +693,19 @@ class TestWorkerConfig(ConfigurationTestSuite):
         """Check that global configuration is set properly"""
         assert orion.core.config.to_dict()["worker"] == self.config["worker"]
 
-        self._mock_consumer(monkeypatch)
-        self._mock_producer(monkeypatch)
-        self._mock_workon(monkeypatch)
+        self._mock(monkeypatch)
 
         command = f"hunt --exp-max-trials 0 -n test python {script} -x~uniform(0,1)"
         orion.core.cli.main(command.split(" "))
 
-        self._check_consumer(self.config["worker"])
-        self._check_producer(self.config["worker"])
-        self._check_workon(self.config["worker"])
+        self._check_mocks(self.config["worker"])
 
     def check_env_var_config(self, tmp_path, monkeypatch):
         """Check that env vars overrides global configuration"""
         env_var_config = {
+            "n_workers": self.env_vars["ORION_N_WORKERS"],
+            "executor": self.env_vars["ORION_EXECUTOR"],
+            "executor_configuration": self.config["worker"]["executor_configuration"],
             "heartbeat": self.env_vars["ORION_HEARTBEAT"],
             "max_trials": self.env_vars["ORION_WORKER_MAX_TRIALS"],
             "max_broken": self.env_vars["ORION_WORKER_MAX_BROKEN"],
@@ -651,16 +716,16 @@ class TestWorkerConfig(ConfigurationTestSuite):
 
         assert orion.core.config.to_dict()["worker"] == env_var_config
 
-        self._mock_consumer(monkeypatch)
-        self._mock_producer(monkeypatch)
-        self._mock_workon(monkeypatch)
+        # Override executor configuration otherwise joblib will fail.
+        orion.core.config.worker.executor_configuration = {}
+        env_var_config["executor_configuration"] = {}
+
+        self._mock(monkeypatch)
 
         command = f"hunt --exp-max-trials 0 -n test python {script} -x~uniform(0,1)"
         orion.core.cli.main(command.split(" "))
 
-        self._check_consumer(env_var_config)
-        self._check_producer(env_var_config)
-        self._check_workon(env_var_config)
+        self._check_mocks(env_var_config)
 
     def check_db_config(self):
         """No Storage config in DB, no test"""
@@ -668,20 +733,22 @@ class TestWorkerConfig(ConfigurationTestSuite):
 
     def check_local_config(self, tmp_path, conf_file, monkeypatch):
         """Check that local configuration overrides global/envvars configuration"""
-        self._mock_consumer(monkeypatch)
-        self._mock_producer(monkeypatch)
-        self._mock_workon(monkeypatch)
+        self._mock(monkeypatch)
+
+        # Override executor so that executor and configuration are coherent in global config
+        os.environ["ORION_EXECUTOR"] = "dask"
 
         command = f"hunt --exp-max-trials 0 -n test -c {conf_file} python {script} -x~uniform(0,1)"
         orion.core.cli.main(command.split(" "))
 
-        self._check_consumer(self.local["worker"])
-        self._check_producer(self.local["worker"])
-        self._check_workon(self.local["worker"])
+        self._check_mocks(self.local["worker"])
 
     def check_cmd_args_config(self, tmp_path, conf_file, monkeypatch):
         """Check that cmdargs configuration overrides global/envvars/local configuration"""
         config = {
+            "n_workers": self.cmdargs["n-workers"],
+            "executor": self.cmdargs["executor"],
+            "executor_configuration": {"threads_per_worker": 2},
             "heartbeat": self.cmdargs["heartbeat"],
             "max_trials": self.cmdargs["worker-max-trials"],
             "max_broken": self.cmdargs["worker-max-broken"],
@@ -690,9 +757,10 @@ class TestWorkerConfig(ConfigurationTestSuite):
             "user_script_config": self.cmdargs["user-script-config"],
         }
 
-        self._mock_consumer(monkeypatch)
-        self._mock_producer(monkeypatch)
-        self._mock_workon(monkeypatch)
+        self._mock(monkeypatch)
+
+        # Override executor so that executor and configuration are coherent in global config
+        os.environ["ORION_EXECUTOR"] = "dask"
 
         command = f"hunt --worker-max-trials 0 -c {conf_file} -n cmd-test"
         command += " " + " ".join(
@@ -701,9 +769,7 @@ class TestWorkerConfig(ConfigurationTestSuite):
         command += f" python {script} -x~uniform(0,1)"
         orion.core.cli.main(command.split(" "))
 
-        self._check_consumer(config)
-        self._check_producer(config)
-        self._check_workon(config)
+        self._check_mocks(config)
 
 
 class TestEVCConfig(ConfigurationTestSuite):
@@ -711,6 +777,7 @@ class TestEVCConfig(ConfigurationTestSuite):
 
     config = {
         "evc": {
+            "enable": False,
             "auto_resolution": False,
             "manual_resolution": True,
             "non_monitored_arguments": ["test", "one"],
@@ -724,6 +791,7 @@ class TestEVCConfig(ConfigurationTestSuite):
     }
 
     env_vars = {
+        "ORION_EVC_ENABLE": "true",
         "ORION_EVC_MANUAL_RESOLUTION": "",
         "ORION_EVC_NON_MONITORED_ARGUMENTS": "test:two:others",
         "ORION_EVC_IGNORE_CODE_CHANGES": "",
@@ -736,9 +804,10 @@ class TestEVCConfig(ConfigurationTestSuite):
 
     local = {
         "evc": {
+            "enable": False,
             "manual_resolution": True,
             "non_monitored_arguments": ["test", "local"],
-            "ignore_code_changes": True,
+            "ignore_code_changes": False,
             "algorithm_change": True,
             "code_change_type": "break",
             "cli_change_type": "break",
@@ -748,9 +817,10 @@ class TestEVCConfig(ConfigurationTestSuite):
     }
 
     cmdargs = {
+        "enable-evc": True,
         "manual-resolution": False,
         "non-monitored-arguments": "test:cmdargs",
-        "ignore-code-changes": False,
+        "ignore-code-changes": True,
         "algorithm-change": False,
         "code-change-type": "noeffect",
         "cli-change-type": "unsure",
@@ -764,13 +834,13 @@ class TestEVCConfig(ConfigurationTestSuite):
 
     def _mock_consumer(self, monkeypatch):
         self.consumer = None
-        old_init = orion.core.worker.Consumer.__init__
+        old_init = orion.core.cli.hunt.Consumer.__init__
 
         def init(c_self, *args, **kwargs):
             old_init(c_self, *args, **kwargs)
             self.consumer = c_self
 
-        monkeypatch.setattr(orion.core.worker.Consumer, "__init__", init)
+        monkeypatch.setattr(orion.core.cli.hunt.Consumer, "__init__", init)
 
     def _check_consumer(self, config):
         assert self.consumer.ignore_code_changes == config["ignore_code_changes"]
@@ -780,37 +850,54 @@ class TestEVCConfig(ConfigurationTestSuite):
         assert orion.core.config.to_dict()["evc"] == self.config["evc"]
 
         name = "global-test"
-        command = (
-            f"hunt --worker-max-trials 0 -n {name} python {script} -x~uniform(0,1)"
-        )
+        command = f"hunt --enable-evc --worker-max-trials 0 -n {name} python {script} -x~uniform(0,1)"
         assert orion.core.cli.main(command.split(" ")) == 0
 
-        # Test that manual_resolution is True and it branches when changing cli
+        # Test that manual_resolution is True and it branches when changing cli (thus crash)
         assert orion.core.cli.main(f"{command} --cli-change ".split(" ")) == 1
 
         command = "hunt --auto-resolution " + command[5:]
 
-        command = self._check_cli_change(
-            name, command, version=1, change_type="noeffect"
-        )
-        command = self._check_non_monitored_arguments(
-            name, command, version=2, non_monitored_arguments=["test", "one"]
+        self._check_enable(name, command.replace(" --enable-evc", ""), enabled=False)
+
+        self._check_cli_change(name, command, change_type="noeffect")
+
+        self._check_non_monitored_arguments(
+            name, command, non_monitored_arguments=["test", "one"]
         )
+
         self._check_script_config_change(
-            tmp_path, name, command, version=2, change_type="noeffect"
-        )
-        self._check_code_change(
-            monkeypatch,
-            name,
-            command,
-            version=3,
-            mock_ignore_code_changes=None,
-            ignore_code_changes=self.config["evc"]["ignore_code_changes"],
-            change_type=self.config["evc"]["code_change_type"],
+            tmp_path, name, command, change_type="noeffect"
         )
 
+        # EVC not enabled, code change should be ignored even if option is set to True
+        assert self.config["evc"]["enable"] is False
+        with monkeypatch.context() as m:
+            self._check_code_change(
+                m,
+                name,
+                command.replace("--enable-evc ", ""),
+                mock_ignore_code_changes=True,
+                ignore_code_changes=True,
+                change_type=self.config["evc"]["code_change_type"],
+                enable_evc=False,
+            )
+
+        # EVC is enabled, option should be honored
+        with monkeypatch.context() as m:
+            self._check_code_change(
+                m,
+                name,
+                command,
+                mock_ignore_code_changes=None,
+                ignore_code_changes=self.config["evc"]["ignore_code_changes"],
+                change_type=self.config["evc"]["code_change_type"],
+                enable_evc=True,
+            )
+
     def check_env_var_config(self, tmp_path, monkeypatch):
         """Check that env vars overrides global configuration"""
+        assert orion.core.config.evc.enable
         assert not orion.core.config.evc.manual_resolution
         assert not orion.core.config.evc.ignore_code_changes
         assert not orion.core.config.evc.algorithm_change
@@ -830,24 +917,42 @@ class TestEVCConfig(ConfigurationTestSuite):
         )
         assert orion.core.cli.main(command.split(" ")) == 0
 
-        # TODO: Anything to test still???
-        command = self._check_cli_change(name, command, version=1, change_type="unsure")
-        command = self._check_non_monitored_arguments(
-            name, command, version=2, non_monitored_arguments=["test", "two", "others"]
-        )
-        self._check_script_config_change(
-            tmp_path, name, command, version=2, change_type="unsure"
-        )
+        self._check_enable(name, command, enabled=True)
 
-        self._check_code_change(
-            monkeypatch,
-            name,
-            command,
-            version=3,
-            mock_ignore_code_changes=None,
-            ignore_code_changes=bool(self.env_vars["ORION_EVC_IGNORE_CODE_CHANGES"]),
-            change_type=self.env_vars["ORION_EVC_CODE_CHANGE"],
+        self._check_cli_change(name, command, change_type="unsure")
+        self._check_non_monitored_arguments(
+            name, command, non_monitored_arguments=["test", "two", "others"]
         )
+        self._check_script_config_change(tmp_path, name, command, change_type="unsure")
+
+        # Enable EVC, ignore_code_changes is False
+        with monkeypatch.context() as m:
+            self._check_code_change(
+                m,
+                name,
+                command,
+                mock_ignore_code_changes=None,
+                ignore_code_changes=bool(
+                    self.env_vars["ORION_EVC_IGNORE_CODE_CHANGES"]
+                ),
+                change_type=self.env_vars["ORION_EVC_CODE_CHANGE"],
+                enable_evc=True,
+            )
+
+        # Disable EVC, ignore_code_changes is True for Consumer
+        os.environ["ORION_EVC_ENABLE"] = ""
+        with monkeypatch.context() as m:
+            self._check_code_change(
+                m,
+                name,
+                command,
+                mock_ignore_code_changes=None,
+                ignore_code_changes=bool(
+                    self.env_vars["ORION_EVC_IGNORE_CODE_CHANGES"]
+                ),
+                change_type=self.env_vars["ORION_EVC_CODE_CHANGE"],
+                enable_evc=False,
+            )
 
     def check_db_config(self):
         """No Storage config in DB, no test"""
@@ -857,7 +962,7 @@ class TestEVCConfig(ConfigurationTestSuite):
         """Check that local configuration overrides global/envvars configuration"""
         name = "local-test"
         command = (
-            f"hunt --worker-max-trials 0 -n {name} -c {conf_file} "
+            f"hunt --enable-evc --worker-max-trials 0 -n {name} -c {conf_file} "
             f"python {script} -x~uniform(0,1)"
         )
 
@@ -868,61 +973,75 @@ class TestEVCConfig(ConfigurationTestSuite):
 
         command = "hunt --auto-resolution " + command[5:]
 
-        command = self._check_cli_change(
-            name, command, version=1, change_type=self.local["evc"]["cli_change_type"]
+        self._check_enable(name, command.replace(" --enable-evc", ""), enabled=False)
+
+        self._check_cli_change(
+            name, command, change_type=self.local["evc"]["cli_change_type"]
         )
-        command = self._check_non_monitored_arguments(
+        self._check_non_monitored_arguments(
             name,
             command,
-            version=2,
             non_monitored_arguments=self.local["evc"]["non_monitored_arguments"],
         )
         self._check_script_config_change(
             tmp_path,
             name,
             command,
-            version=2,
             change_type=self.local["evc"]["config_change_type"],
         )
-        self._check_code_change(
-            monkeypatch,
-            name,
-            command,
-            version=3,
-            mock_ignore_code_changes=True,
-            ignore_code_changes=self.local["evc"]["ignore_code_changes"],
-            change_type=self.local["evc"]["code_change_type"],
-        )
+
+        # enabled evc, ignore code changes so to True
+        with monkeypatch.context() as m:
+            self._check_code_change(
+                m,
+                name,
+                command,
+                mock_ignore_code_changes=False,
+                ignore_code_changes=self.local["evc"]["ignore_code_changes"],
+                change_type=self.local["evc"]["code_change_type"],
+                enable_evc=True,
+            )
+
+        # disabled evc, ignore code changes so to True
+        with monkeypatch.context() as m:
+            self._check_code_change(
+                m,
+                name,
+                command.replace("--enable-evc ", ""),
+                mock_ignore_code_changes=False,
+                ignore_code_changes=self.local["evc"]["ignore_code_changes"],
+                change_type=self.local["evc"]["code_change_type"],
+                enable_evc=False,
+            )
 
     def check_cmd_args_config(self, tmp_path, conf_file, monkeypatch):
         """Check that cmdargs configuration overrides global/envvars/local configuration"""
         name = "cmd-test"
         command = (
             f"hunt --worker-max-trials 0 -c {conf_file} -n {name} "
+            "--enable-evc "
             "--auto-resolution "
-            "--non-monitored-arguments test:cmdargs "
-            "--code-change-type noeffect "
-            "--cli-change-type unsure "
-            "--config-change-type break "
             f"python {script} -x~uniform(0,1)"
         )
         assert orion.core.cli.main(command.split(" ")) == 0
 
-        command = self._check_cli_change(
-            name, command, version=1, change_type=self.cmdargs["cli-change-type"]
-        )
-        command = self._check_non_monitored_arguments(
+        self._check_enable(name, command, enabled=True)
+
+        self._check_cli_change(
             name,
-            command,
-            version=2,
+            command="hunt --cli-change-type unsure " + command[5:],
+            change_type=self.cmdargs["cli-change-type"],
+        )
+        self._check_non_monitored_arguments(
+            name,
+            command="hunt --non-monitored-arguments test:cmdargs " + command[5:],
             non_monitored_arguments=self.cmdargs["non-monitored-arguments"].split(":"),
         )
 
         self._check_script_config_change(
             tmp_path,
             name,
-            command,
-            version=2,
+            command="hunt --config-change-type break " + command[5:],
             change_type=self.cmdargs["config-change-type"],
         )
 
@@ -937,68 +1056,95 @@ class TestEVCConfig(ConfigurationTestSuite):
         monkeypatch.setattr(orion.core.io.resolve_config, "fetch_config", mock_local)
 
         # Check that ignore_code_changes is rightly False
-        self._check_code_change(
-            monkeypatch,
-            name,
-            command,
-            version=3,
-            mock_ignore_code_changes=False,
-            ignore_code_changes=False,
-            change_type=self.cmdargs["code-change-type"],
-        )
+        with monkeypatch.context() as m:
+            self._check_code_change(
+                m,
+                name,
+                command="hunt --code-change-type noeffect " + command[5:],
+                mock_ignore_code_changes=False,
+                ignore_code_changes=False,
+                change_type=self.cmdargs["code-change-type"],
+                enable_evc=True,
+            )
 
-        command = "hunt --ignore-code-changes " + command[5:]
+        # Check that ignore_code_changes is now True because --ignore-code-changes was added
+        with monkeypatch.context() as m:
+            self._check_code_change(
+                m,
+                name,
+                command="hunt --ignore-code-changes --code-change-type noeffect "
+                + command[5:],
+                mock_ignore_code_changes=True,
+                ignore_code_changes=True,
+                change_type=self.cmdargs["code-change-type"],
+                enable_evc=True,
+            )
 
-        # Check that ignore_code_changes is now True
-        self._check_code_change(
-            monkeypatch,
-            name,
-            command,
-            version=4,
-            mock_ignore_code_changes=True,
-            ignore_code_changes=True,
-            change_type=self.cmdargs["code-change-type"],
-        )
+        # Check that ignore_code_changes is forced to True in consumer
+        # even if --ignore-code-changes is not passed
+        with monkeypatch.context() as m:
+            self._check_code_change(
+                m,
+                name,
+                command.replace("--enable-evc ", ""),
+                mock_ignore_code_changes=False,
+                ignore_code_changes=False,
+                change_type=self.cmdargs["code-change-type"],
+                enable_evc=False,
+            )
 
-    def _check_cli_change(self, name, command, version, change_type):
-        command += " --cli-change"
+    @with_storage_fork
+    def _check_enable(self, name, command, enabled):
+        command += " --cli-change "
+        experiment = get_experiment(name)
+        if enabled:
+            assert orion.core.cli.main(command.split(" ")) == 0
+            assert get_experiment(name).version == experiment.version + 1
+        else:
+            assert orion.core.cli.main(command.split(" ")) == 0
+            assert get_experiment(name).version == experiment.version
 
+    @with_storage_fork
+    def _check_cli_change(self, name, command, change_type):
+        command += " --cli-change "
+
+        experiment = get_experiment(name)
         # Test that manual_resolution is False and it branches when changing cli
         assert orion.core.cli.main(command.split(" ")) == 0
 
-        experiment = get_experiment(name, version=version + 1)
-        assert experiment.version == version + 1
-        assert experiment.refers["adapter"].configuration[0] == {
+        new_experiment = get_experiment(name)
+
+        assert new_experiment.version == experiment.version + 1
+        assert new_experiment.refers["adapter"].configuration[0] == {
             "of_type": "commandlinechange",
             "change_type": change_type,
         }
 
-        return command
-
-    def _check_non_monitored_arguments(
-        self, name, command, version, non_monitored_arguments
-    ):
+    @with_storage_fork
+    def _check_non_monitored_arguments(self, name, command, non_monitored_arguments):
         for argument in non_monitored_arguments:
             command += f" --{argument} "
 
+        experiment = get_experiment(name)
         # Test that cli change with non-monitored args do not cause branching
         assert orion.core.cli.main(command.split(" ")) == 0
 
-        experiment = get_experiment(name, version=version + 1)
-        assert experiment.version == version
-
-        return command
+        assert get_experiment(name).version == experiment.version
 
+    @with_storage_fork
     def _check_code_change(
         self,
         monkeypatch,
         name,
         command,
-        version,
         mock_ignore_code_changes,
         ignore_code_changes,
         change_type,
+        enable_evc,
     ):
+        """Check if code changes are correctly ignored during experiment build and by consumer
+        between two trial executions.
+        """
 
         # Test that code change is handled with 'no-effect'
         def fixed_dictionary(user_script):
@@ -1024,27 +1170,40 @@ class TestEVCConfig(ConfigurationTestSuite):
                 assert (
                     branching_config["ignore_code_changes"] is mock_ignore_code_changes
                 )
-                branching_config["ignore_code_changes"] = False
+            # branching_config["ignore_code_changes"] = False
             return detect(old_config, new_config, branching_config)
 
         monkeypatch.setattr(
             orion.core.evc.conflicts.CodeConflict, "detect", mock_detect
         )
+
+        experiment = get_experiment(name)
+
         assert orion.core.cli.main(command.split(" ")) == 0
-        self._check_consumer({"ignore_code_changes": ignore_code_changes})
+        self._check_consumer(
+            {
+                "ignore_code_changes": (
+                    (enable_evc and ignore_code_changes) or not enable_evc
+                )
+            }
+        )
 
-        experiment = get_experiment(name, version=version + 1)
-        assert experiment.version == version + 1
-        assert experiment.refers["adapter"].configuration[0] == {
-            "of_type": "codechange",
-            "change_type": change_type,
-        }
+        new_experiment = get_experiment(name)
+        if enable_evc and not ignore_code_changes:
+            assert new_experiment.version == experiment.version + 1
+            assert new_experiment.refers["adapter"].configuration[0] == {
+                "of_type": "codechange",
+                "change_type": change_type,
+            }
+        elif enable_evc:  # But code change ignored, so no branching event.
+            assert new_experiment.version == experiment.version
+        else:
+            assert new_experiment.version == experiment.version
 
-        monkeypatch.undo()
+    @with_storage_fork
+    def _check_script_config_change(self, tmp_path, name, command, change_type):
 
-    def _check_script_config_change(
-        self, tmp_path, name, command, version, change_type
-    ):
+        experiment = get_experiment(name)
 
         # Test that config change is handled with 'break'
         with self.setup_user_script_config(tmp_path) as user_script_config:
@@ -1052,11 +1211,11 @@ class TestEVCConfig(ConfigurationTestSuite):
             command += f" --config {user_script_config}"
             assert orion.core.cli.main(command.split(" ")) == 0
 
-        experiment = get_experiment(name, version=version + 1)
-        assert experiment.version == version + 1
-        print(experiment.refers["adapter"].configuration)
-        assert len(experiment.refers["adapter"].configuration) == 2
-        assert experiment.refers["adapter"].configuration[1] == {
+        new_experiment = get_experiment(name)
+
+        assert new_experiment.version == experiment.version + 1
+        assert len(new_experiment.refers["adapter"].configuration) == 2
+        assert new_experiment.refers["adapter"].configuration[1] == {
             "of_type": "scriptconfigchange",
             "change_type": change_type,
         }
