@@ -11,32 +11,26 @@ from orion.client import report_results
 def function(x):
     """Evaluate partial information of a quadratic."""
     z = x - 34.56789
-    return 4 * z**2 + 23.4, 8 * z
+    return 4 * z ** 2 + 23.4, 8 * z
 
 
 def execute():
     """Execute a simple pipeline as an example."""
     # 1. Receive inputs as you want
     parser = argparse.ArgumentParser()
-    parser.add_argument('--config', required=True)
+    parser.add_argument("--config", required=True)
     inputs = parser.parse_args()
 
-    with open(inputs.config, 'r') as f:
+    with open(inputs.config, "r") as f:
         config = yaml.safe_load(f)
 
     # 2. Perform computations
-    y, dy = function(config['x'])
+    y, dy = function(config["x"])
 
     # 3. Gather and report results
     results = list()
-    results.append(dict(
-        name='example_objective',
-        type='objective',
-        value=y))
-    results.append(dict(
-        name='example_gradient',
-        type='gradient',
-        value=[dy]))
+    results.append(dict(name="example_objective", type="objective", value=y))
+    results.append(dict(name="example_gradient", type="gradient", value=[dy]))
 
     report_results(results)
 
