@@ -18,7 +18,7 @@ class Gradient_Descent(BaseAlgorithm):
 
     requires = "real"
 
-    def __init__(self, space, learning_rate=1.0, dx_tolerance=1e-7):
+    def __init__(self, space, learning_rate=1.0, dx_tolerance=1e-5):
         """Declare `learning_rate` as a hyperparameter of this algorithm."""
         super(Gradient_Descent, self).__init__(
             space, learning_rate=learning_rate, dx_tolerance=dx_tolerance
@@ -27,13 +27,14 @@ class Gradient_Descent(BaseAlgorithm):
         self.current_point = None
         self.gradient = numpy.array([numpy.inf])
 
-    def suggest(self, num=1):
+    def suggest(self, num):
         """Suggest a `num`ber of new sets of parameters.
 
         Perform a step towards negative gradient and suggest that point.
 
         """
-        assert num == 1  # Simple gradient descent does not make sense else
+        num = 1  # Simple gradient descent only make sense for 1 point at a time.
+
         if not self.has_observed_once:
             return self.space.sample(1)
 
