@@ -1,6 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 """Tests for :mod:`orion.algo.tpe`."""
+import itertools
 
 import numpy
 import pytest
@@ -16,6 +17,7 @@ from orion.algo.tpe import (
     ramp_up_weights,
 )
 from orion.core.worker.transformer import build_required_space
+from orion.testing.algo import BaseAlgoTests
 
 
 @pytest.fixture()
@@ -679,3 +681,103 @@ class TestTPE:
         tpe.n_ei_candidates = 24
         point = tpe.suggest(1)
         assert len(point) > 0
+
+
+N_INIT = 10
+
+
+class TestTPE(BaseAlgoTests):
+    algo_name = "tpe"
+    config = {
+        "seed": 123456,
+        "n_initial_points": N_INIT,
+        "n_ei_candidates": 4,
+        "gamma": 0.5,
+        "equal_weight": True,
+        "prior_weight": 0.8,
+        "full_weight_num": 10,
+    }
+
+    def test_suggest_init(self, mocker):
+        algo = self.create_algo()
+        spy = self.spy_phase(mocker, 0, algo, "space.sample")
+        points = algo.suggest(1000)
+        assert len(points) == N_INIT
+
+    def test_suggest_init_missing(self, mocker):
+        algo = self.create_algo()
+        missing = 3
+        spy = self.spy_phase(mocker, N_INIT - missing, algo, "space.sample")
+        points = algo.suggest(1000)
+        assert len(points) == missing
+
+    def test_suggest_init_overflow(self, mocker):
+        algo = self.create_algo()
+        spy = self.spy_phase(mocker, N_INIT - 1, algo, "space.sample")
+        # Now reaching N_INIT
+        points = algo.suggest(1000)
+        assert len(points) == 1
+        # Verify point was sampled randomly, not using BO
+        assert spy.call_count == 1
+        # Overflow above N_INIT
+        points = algo.suggest(1000)
+        assert len(points) == 1
+        # Verify point was sampled randomly, not using BO
+        assert spy.call_count == 2
+
+    def test_suggest_n(self, mocker, num, attr):
+        """Verify that suggest returns correct number of trials if ``num`` is specified in ``suggest``."""
+        algo = self.create_algo()
+        spy = self.spy_phase(mocker, num, algo, attr)
+        points = algo.suggest(5)
+        if num == 0:
+            assert len(points) == 5
+        else:
+            assert len(points) == 1
+
+    def test_thin_real_space(self, monkeypatch):
+        algo = self.create_algo()
+        self.force_observe(N_INIT + 1, algo)
+
+        original_sample = GMMSampler.sample
+
+        low = 0.5
+        high = 0.50001
+
+        def sample(self, num):
+            self.low = low
+            self.high = high
+            return original_sample(self, num)
+
+        monkeypatch.setattr(GMMSampler, "sample", sample)
+        with pytest.raises(RuntimeError) as exc:
+            algo.suggest(1)
+
+        assert exc.match(f"Failed to sample in interval \({low}, {high}\)")
+
+    def test_is_done_cardinality(self):
+        # TODO: Support correctly loguniform(discrete=True)
+        #       See https://github.com/Epistimio/orion/issues/566
+        space = self.update_space(
+            {
+                "x": "uniform(0, 4, discrete=True)",
+                "y": "choices(['a', 'b', 'c'])",
+                "z": "uniform(1, 6, discrete=True)",
+            }
+        )
+        space = self.create_space(space)
+        assert space.cardinality == 5 * 3 * 6
+
+        algo = self.create_algo(space=space)
+        for i, (x, y, z) in enumerate(itertools.product(range(5), "abc", range(1, 7))):
+            assert not algo.is_done
+            n = algo.n_suggested
+            algo.observe([[x, y, z]], [dict(objective=i)])
+            assert algo.n_suggested == n + 1
+
+        assert i + 1 == space.cardinality
+
+        assert algo.is_done
+
+
+TestTPE.set_phases([("random", 0, "space.sample"), ("bo", N_INIT + 1, "_suggest_bo")])
