@@ -266,7 +266,9 @@ class TestCreateExperiment:
         """Test creating a differing experiment that cause branching."""
         with OrionState(experiments=[config]):
             experiment = create_experiment(
-                config["name"], space={"y": "uniform(0, 10)"}
+                config["name"],
+                space={"y": "uniform(0, 10)"},
+                branching={"enable": True},
             )
 
             assert experiment.name == config["name"]
@@ -289,7 +291,11 @@ class TestCreateExperiment:
         """
         with OrionState(experiments=[config]):
             parent = create_experiment(config["name"])
-            child = create_experiment(config["name"], space={"y": "uniform(0, 10)"})
+            child = create_experiment(
+                config["name"],
+                space={"y": "uniform(0, 10)"},
+                branching={"enable": True},
+            )
 
             def insert_race_condition(self, query):
                 is_auto_version_query = query == {
@@ -315,7 +321,9 @@ class TestCreateExperiment:
             )
 
             experiment = create_experiment(
-                config["name"], space={"y": "uniform(0, 10)"}
+                config["name"],
+                space={"y": "uniform(0, 10)"},
+                branching={"enable": True},
             )
 
             assert insert_race_condition.count == 1
@@ -326,7 +334,11 @@ class TestCreateExperiment:
         """Test that two or more race condition leads to raise"""
         with OrionState(experiments=[config]):
             parent = create_experiment(config["name"])
-            child = create_experiment(config["name"], space={"y": "uniform(0, 10)"})
+            child = create_experiment(
+                config["name"],
+                space={"y": "uniform(0, 10)"},
+                branching={"enable": True},
+            )
 
             def insert_race_condition(self, query):
                 is_auto_version_query = query == {
@@ -350,7 +362,11 @@ class TestCreateExperiment:
             )
 
             with pytest.raises(RaceCondition) as exc:
-                create_experiment(config["name"], space={"y": "uniform(0, 10)"})
+                create_experiment(
+                    config["name"],
+                    space={"y": "uniform(0, 10)"},
+                    branching={"enable": True},
+                )
 
             assert insert_race_condition.count == 2
             assert "There was a race condition during branching and new version" in str(
@@ -361,10 +377,17 @@ class TestCreateExperiment:
         """Test creating a differing experiment that cause branching."""
         new_space = {"y": "uniform(0, 10)"}
         with OrionState(experiments=[config]):
-            create_experiment(config["name"], space=new_space)
+            create_experiment(
+                config["name"], space=new_space, branching={"enable": True}
+            )
 
             with pytest.raises(BranchingEvent) as exc:
-                create_experiment(config["name"], version=1, space=new_space)
+                create_experiment(
+                    config["name"],
+                    version=1,
+                    space=new_space,
+                    branching={"enable": True},
+                )
 
             assert "Configuration is different and generates" in str(exc.value)
 
@@ -503,6 +526,30 @@ class TestWorkon:
         assert experiment2.name == "voici"
         assert len(experiment2.fetch_trials()) == 1
 
+    def test_workon_with_parallel_backend(self):
+        """Test there is no impact of joblib parallel for workon function"""
+
+        def foo(x):
+            return [dict(name="result", type="objective", value=x * 2)]
+
+        import joblib
+
+        with joblib.parallel_backend("loky"):
+            experiment = workon(
+                foo, space={"x": "uniform(0, 10)"}, max_trials=5, name="voici"
+            )
+
+        assert experiment.name == "voici"
+        assert len(experiment.fetch_trials()) == 5
+
+        with joblib.parallel_backend("loky", n_jobs=-1):
+            experiment = workon(
+                foo, space={"x": "uniform(0, 10)"}, max_trials=3, name="voici"
+            )
+
+        assert experiment.name == "voici"
+        assert len(experiment.fetch_trials()) == 3
+
 
 class TestGetExperiment:
     """Test :meth:`orion.client.get_experiment`"""
