@@ -81,7 +81,7 @@ def conflicts(
 @pytest.fixture
 def branch_builder(conflicts):
     """Generate the experiment branch builder"""
-    return ExperimentBranchBuilder(conflicts, {"manual_resolution": True})
+    return ExperimentBranchBuilder(conflicts, manual_resolution=True)
 
 
 @pytest.fixture
