@@ -3,6 +3,7 @@
 """Example usage and tests for :mod:`orion.core.io.experiment_builder`."""
 import copy
 import datetime
+import logging
 
 import pytest
 
@@ -322,20 +323,20 @@ def test_build_from_args_no_hit(config_file, random_dt, script_path, new_config)
 
         exp = experiment_builder.build_from_args(cmdargs)
 
-    assert exp.name == cmdargs["name"]
-    assert exp.configuration["refers"] == {
-        "adapter": [],
-        "parent_id": None,
-        "root_id": exp._id,
-    }
-    assert exp.metadata["datetime"] == random_dt
-    assert exp.metadata["user"] == "dendi"
-    assert exp.metadata["user_script"] == cmdargs["user_args"][0]
-    assert exp.metadata["user_args"] == cmdargs["user_args"]
-    assert exp.pool_size == 1
-    assert exp.max_trials == 100
-    assert exp.max_broken == 5
-    assert exp.algorithms.configuration == {"random": {"seed": None}}
+        assert exp.name == cmdargs["name"]
+        assert exp.configuration["refers"] == {
+            "adapter": [],
+            "parent_id": None,
+            "root_id": exp._id,
+        }
+        assert exp.metadata["datetime"] == random_dt
+        assert exp.metadata["user"] == "dendi"
+        assert exp.metadata["user_script"] == cmdargs["user_args"][0]
+        assert exp.metadata["user_args"] == cmdargs["user_args"]
+        assert exp.pool_size == 1
+        assert exp.max_trials == 100
+        assert exp.max_broken == 5
+        assert exp.algorithms.configuration == {"random": {"seed": None}}
 
 
 @pytest.mark.usefixtures(
@@ -453,22 +454,22 @@ def test_build_no_hit(config_file, random_dt, script_path):
             name, space=space, max_trials=max_trials, max_broken=max_broken
         )
 
-    assert exp.name == name
-    assert exp.configuration["refers"] == {
-        "adapter": [],
-        "parent_id": None,
-        "root_id": exp._id,
-    }
-    assert exp.metadata == {
-        "datetime": random_dt,
-        "user": "tsirif",
-        "orion_version": "XYZ",
-    }
-    assert exp.configuration["space"] == space
-    assert exp.max_trials == max_trials
-    assert exp.max_broken == max_broken
-    assert not exp.is_done
-    assert exp.algorithms.configuration == {"random": {"seed": None}}
+        assert exp.name == name
+        assert exp.configuration["refers"] == {
+            "adapter": [],
+            "parent_id": None,
+            "root_id": exp._id,
+        }
+        assert exp.metadata == {
+            "datetime": random_dt,
+            "user": "tsirif",
+            "orion_version": "XYZ",
+        }
+        assert exp.configuration["space"] == space
+        assert exp.max_trials == max_trials
+        assert exp.max_broken == max_broken
+        assert not exp.is_done
+        assert exp.algorithms.configuration == {"random": {"seed": None}}
 
 
 def test_build_no_commandline_config():
@@ -546,7 +547,9 @@ def test_build_from_args_without_cmd(old_config_file, script_path, new_config):
     assert exp.algorithms.configuration == new_config["algorithms"]
 
 
-@pytest.mark.usefixtures("with_user_tsirif", "version_XYZ")
+@pytest.mark.usefixtures(
+    "with_user_tsirif", "version_XYZ", "mock_infer_versioning_metadata"
+)
 class TestExperimentVersioning(object):
     """Create new Experiment with auto-versioning."""
 
@@ -566,12 +569,46 @@ class TestExperimentVersioning(object):
 
         assert exp.version == 1
 
+    def test_experiment_overwritten_evc_disabled(self, parent_version_config, caplog):
+        """Build an existing experiment with different config, overwritting previous config."""
+        parent_version_config.pop("version")
+        with OrionState(experiments=[parent_version_config]):
+
+            with caplog.at_level(logging.WARNING):
+
+                exp = experiment_builder.build(name=parent_version_config["name"])
+                assert "Running experiment in a different state" not in caplog.text
+
+            assert exp.version == 1
+            assert exp.configuration["algorithms"] == {"random": {"seed": None}}
+
+            with caplog.at_level(logging.WARNING):
+
+                exp = experiment_builder.build(
+                    name=parent_version_config["name"], algorithms="gradient_descent"
+                )
+                assert "Running experiment in a different state" in caplog.text
+
+            assert exp.version == 1
+            assert list(exp.configuration["algorithms"].keys())[0] == "gradient_descent"
+
+            caplog.clear()
+            with caplog.at_level(logging.WARNING):
+
+                exp = experiment_builder.load(name=parent_version_config["name"])
+                assert "Running experiment in a different state" not in caplog.text
+
+            assert exp.version == 1
+            assert list(exp.configuration["algorithms"].keys())[0] == "gradient_descent"
+
     def test_backward_compatibility_no_version(self, parent_version_config):
         """Branch from parent that has no version field."""
         parent_version_config.pop("version")
         with OrionState(experiments=[parent_version_config]):
             exp = experiment_builder.build(
-                name=parent_version_config["name"], space={"y": "uniform(0, 10)"}
+                name=parent_version_config["name"],
+                space={"y": "uniform(0, 10)"},
+                branching={"enable": True},
             )
 
         assert exp.version == 2
@@ -854,7 +891,7 @@ class TestBuild(object):
             child_name = "child"
 
             child = experiment_builder.build(
-                name=name, branching={"branch_to": child_name}
+                name=name, branching={"branch_to": child_name, "enable": True}
             )
 
             assert child.name == child_name
@@ -864,7 +901,7 @@ class TestBuild(object):
             child_name = "child2"
 
             child = experiment_builder.build(
-                name=child_name, branching={"branch_from": name}
+                name=child_name, branching={"branch_from": name, "enable": True}
             )
 
             assert child.name == child_name
@@ -878,14 +915,19 @@ class TestBuild(object):
 
         with OrionState(experiments=[], trials=[]):
             parent = experiment_builder.build(name=name, space=space)
-            child = experiment_builder.build(name=name, space={"x": "loguniform(1,10)"})
+            child = experiment_builder.build(
+                name=name, space={"x": "loguniform(1,10)"}, branching={"enable": True}
+            )
             assert child.name == parent.name
             assert parent.version == 1
             assert child.version == 2
 
             with pytest.raises(BranchingEvent) as exc_info:
                 experiment_builder.build(
-                    name=name, version=1, space={"x": "loguniform(1,10)"}
+                    name=name,
+                    version=1,
+                    space={"x": "loguniform(1,10)"},
+                    branching={"enable": True},
                 )
             assert "Configuration is different and generates a branching" in str(
                 exc_info.value
@@ -900,7 +942,9 @@ class TestBuild(object):
 
         with OrionState(experiments=[], trials=[]):
             parent = experiment_builder.build(name, space=space)
-            child = experiment_builder.build(name=name, space={"x": "loguniform(1,10)"})
+            child = experiment_builder.build(
+                name=name, space={"x": "loguniform(1,10)"}, branching={"enable": True}
+            )
             assert child.name == parent.name
             assert parent.version == 1
             assert child.version == 2
@@ -939,7 +983,11 @@ class TestBuild(object):
             )
 
             with pytest.raises(RaceCondition) as exc_info:
-                experiment_builder.build(name=name, space={"x": "loguniform(1,10)"})
+                experiment_builder.build(
+                    name=name,
+                    space={"x": "loguniform(1,10)"},
+                    branching={"enable": True},
+                )
             assert "There was likely a race condition during version" in str(
                 exc_info.value
             )
@@ -968,7 +1016,11 @@ class TestBuild(object):
             )
 
             with pytest.raises(RaceCondition) as exc_info:
-                experiment_builder.build(name=name, space={"x": "loguniform(1,10)"})
+                experiment_builder.build(
+                    name=name,
+                    space={"x": "loguniform(1,10)"},
+                    branching={"enable": True},
+                )
             assert "There was a race condition during branching." in str(exc_info.value)
 
     def test_race_condition_w_version(self, monkeypatch):
@@ -985,7 +1037,9 @@ class TestBuild(object):
 
         with OrionState(experiments=[], trials=[]):
             parent = experiment_builder.build(name, space=space)
-            child = experiment_builder.build(name=name, space={"x": "loguniform(1,10)"})
+            child = experiment_builder.build(
+                name=name, space={"x": "loguniform(1,10)"}, branching={"enable": True}
+            )
             assert child.name == parent.name
             assert parent.version == 1
             assert child.version == 2
@@ -1025,7 +1079,10 @@ class TestBuild(object):
 
             with pytest.raises(BranchingEvent) as exc_info:
                 experiment_builder.build(
-                    name=name, version=1, space={"x": "loguniform(1,10)"}
+                    name=name,
+                    version=1,
+                    space={"x": "loguniform(1,10)"},
+                    branching={"enable": True},
                 )
             assert "Configuration is different and generates" in str(exc_info.value)
 
@@ -1054,7 +1111,10 @@ class TestBuild(object):
 
             with pytest.raises(RaceCondition) as exc_info:
                 experiment_builder.build(
-                    name=name, version=1, space={"x": "loguniform(1,10)"}
+                    name=name,
+                    version=1,
+                    space={"x": "loguniform(1,10)"},
+                    branching={"enable": True},
                 )
             assert "There was a race condition during branching." in str(exc_info.value)
 
