@@ -7,6 +7,7 @@ import datetime
 import inspect
 import json
 import logging
+import pickle
 import tempfile
 
 import pandas
@@ -427,6 +428,22 @@ def test_fetch_all_trials():
         assert trials == cfg.trials
 
 
+def test_fetch_pending_trials():
+    """Fetch a list of the trials that are pending
+
+    trials.status in ['new', 'interrupted', 'suspended']
+    """
+    pending_stati = ["new", "interrupted", "suspended"]
+    stati = pending_stati + ["completed", "broken", "reserved"]
+    with OrionState(trials=generate_trials(stati)) as cfg:
+        exp = Experiment("supernaekei", mode="x")
+        exp._id = cfg.trials[0]["experiment"]
+
+        trials = exp.fetch_pending_trials()
+        assert len(trials) == 3
+        assert set(trial.status for trial in trials) == set(pending_stati)
+
+
 def test_fetch_non_completed_trials():
     """Fetch a list of the trials that are not completed
 
@@ -547,9 +564,31 @@ def test_experiment_stats():
         assert len(stats) == 6
 
 
+def test_experiment_pickleable():
+    """Test experiment instance is pickleable"""
+
+    with OrionState(trials=generate_trials(["new"])) as cfg:
+        exp = Experiment("supernaekei", mode="x")
+        exp._id = cfg.trials[0]["experiment"]
+
+        exp_trials = exp.fetch_trials()
+
+        assert len(exp_trials) > 0
+
+        exp_bytes = pickle.dumps(exp)
+
+        new_exp = pickle.loads(exp_bytes)
+
+        assert [trial.to_dict() for trial in exp_trials] == [
+            trial.to_dict() for trial in new_exp.fetch_trials()
+        ]
+
+
 read_only_methods = [
     "algorithms",
     "configuration",
+    "fetch_lost_trials",
+    "fetch_pending_trials",
     "fetch_noncompleted_trials",
     "fetch_trials",
     "fetch_trials_by_status",
@@ -577,6 +616,7 @@ read_write_only_methods = [
     "register_trial",
     "set_trial_status",
     "update_completed_trial",
+    "duplicate_pending_trials",
 ]
 execute_only_methods = [
     "reserve_trial",
