@@ -5,6 +5,8 @@
 import copy
 import datetime
 import logging
+import os
+import pickle
 import time
 
 import pytest
@@ -157,7 +159,7 @@ def test_setup_storage_custom():
     storage = Storage()
     assert isinstance(storage, Legacy)
     assert isinstance(storage._db, PickledDB)
-    assert storage._db.host == "test.pkl"
+    assert storage._db.host == os.path.abspath("test.pkl")
 
 
 def test_setup_storage_custom_type_missing():
@@ -167,7 +169,7 @@ def test_setup_storage_custom_type_missing():
     storage = Storage()
     assert isinstance(storage, Legacy)
     assert isinstance(storage._db, PickledDB)
-    assert storage._db.host == "test.pkl"
+    assert storage._db.host == os.path.abspath("test.pkl")
 
 
 @pytest.mark.usefixtures("setup_pickleddb_database")
@@ -815,3 +817,13 @@ class TestStorage:
                 assert (
                     trial3.heartbeat is None
                 ), "Legacy does not update trials with a status different from reserved"
+
+    def test_serializable(self, storage):
+        """Test storage can be serialized"""
+        with OrionState(
+            experiments=[base_experiment], trials=generate_trials(), storage=storage
+        ) as cfg:
+            storage = cfg.storage()
+            serialized = pickle.dumps(storage)
+            deserialized = pickle.loads(serialized)
+            assert storage.fetch_experiments({}) == deserialized.fetch_experiments({})
